@@ -22,9 +22,9 @@ int main(int argc, char **argv)
   
   if(argc != 5)
     {  
-       printf("sendpees by willdamn <willdamn@gmail.com>\n");
-       printf("usage: %s <inf> <key_length> <prefix> <victim>\n", argv[0]);
-       printf("Send SEND neighbor solicitation messages and make target to verify a lota CGA and RSA signatures\n");
+       printf("sendpees6 by willdamn <willdamn@gmail.com>\n\n");
+       printf("usage: %s <inf> <key_length> <prefix> <victim>\n\n", argv[0]);
+       printf("Send SEND neighbor solicitation messages and make target to verify a lota CGA and RSA signatures\n\n");
        exit(1);
     }
   
