@@ -1,10 +1,12 @@
 /*
- * (c) 2006 by van Hauser / THC <vh@thc.org>
+ * (c) 2010 by van Hauser / THC <vh@thc.org>
  *
  * THC IPv6 Attack Library
  *
  * Functions: see README
  *
+ * The GPLv3 applies to this code, see the LICENSE file
+ *
  */
 
 
@@ -15,7 +17,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <ctype.h>
-#include <linux/string.h>
+//#include <linux/string.h>
 /* network */
 #include <signal.h>
 #include <sys/stat.h>
@@ -72,7 +74,7 @@ int thc_pcap_function(char *interface, unsigned char *capture, char * function)
   struct bpf_program fcode;
   if (interface == NULL)
     interface = default_interface;
-  if ((pcap_link = pcap_open_live(interface, 65535, 1, -1, errbuf)) == NULL)
+  if ((pcap_link = pcap_open_live(interface, 65535, 0, -1, errbuf)) == NULL)
     return -1;
   if (pcap_compile(pcap_link, &fcode, capture, 1, 0) < 0)
     return -2;
@@ -88,7 +90,7 @@ pcap_t *thc_pcap_init(char *interface, unsigned char *capture) {
   struct bpf_program fcode;
   if (interface == NULL)
     interface = default_interface;
-  if ((pcap_link = pcap_open_live(interface, 65535, 1, -1, errbuf)) == NULL)
+  if ((pcap_link = pcap_open_live(interface, 65535, 0, -1, errbuf)) == NULL)
     return NULL;
   if (pcap_compile(pcap_link, &fcode, capture, 1, 0) < 0)
     return NULL;
@@ -108,6 +110,46 @@ void thc_pcap_close(pcap_t *pcap_link) {
     pcap_close(pcap_link);
 }
 
+/* wow, ugly, conmplicated work for something a standard linux library could do as well :-) */
+void thc_notation2beauty(unsigned char *ipv6) {
+  char buf[40], buf2[40] = ":0:0:", *ptr, *ptr2 = NULL;
+  int i, j, k = 0, l = 0;
+
+  if (ipv6[39] != 0 || strlen(ipv6) != 39)
+    return;
+  
+  memset(buf, 0, sizeof(buf));
+  for (i = 0; i < 8; i++) {
+    ptr = ipv6 + i * 4 + i;
+    j = 0;
+    while (*ptr == '0' && j < 3) {
+      ptr++;
+      j++;
+    }
+    memcpy(&buf[k], ptr, 4 - j);
+    k += 4 - j;
+    buf[k++] = ':';
+  }
+  buf[k-1] = 0;
+  while ((ptr=strstr(buf, buf2)) != NULL) {
+   ptr2 = ptr;
+   strcat(buf2, "0:");
+  }
+  if (ptr2 != NULL) {
+    buf2[strlen(buf2) - 2] = 0;
+    memset(ipv6, 0, 40);
+    if (buf + 1 == ptr2 && buf[0] == '0') {
+      ipv6[0] = ':';
+      l = -1;
+    } else
+      memcpy(ipv6, buf, ptr2 - buf + 1);
+    memcpy(ipv6 + (ptr2 - buf + 1 + l), ptr2 + strlen(buf2) - 1, strlen(buf) - (ptr2 - buf) - strlen(buf2) + 1);
+    if (ipv6[strlen(ipv6)-1] == '0' && ipv6[strlen(ipv6)-2] == ':')
+      ipv6[strlen(ipv6)-1] = 0;
+  } else
+    strcpy(ipv6, buf);
+}
+
 unsigned char *thc_ipv62string(unsigned char *ipv6) {
   char *string = malloc(33);
   int a;
@@ -158,6 +200,7 @@ unsigned char *thc_string2notation(unsigned char *string) {
   } else
     return NULL;
   
+  thc_notation2beauty(notation);
   return notation;
 }
 
@@ -186,7 +229,7 @@ int checksum_pseudo_header(unsigned char *src, unsigned char *dst, unsigned char
   
   if (ptr == NULL)
     return -1;
-  
+
   memset(&ptr, 0, 40 + length);
   memcpy(&ptr[0], src, 16);
   memcpy(&ptr[16], dst, 16);
@@ -201,6 +244,9 @@ int checksum_pseudo_header(unsigned char *src, unsigned char *dst, unsigned char
   if (type == NXT_UDP && checksum == 0)
     checksum=65535;
   
+  if (debug)
+    fprintf(stderr, "Checksum: %d = %p, %p, %d, %p, %d\n", checksum, src, dst, type, data, length);
+  
   return checksum;
 }
 
@@ -234,6 +280,27 @@ char *thc_resolve6(unsigned char *target) {
   return ret_addr;
 }
 
+int thc_get_mtu(char *interface) {
+  int s;
+  struct ifreq ifr;
+
+  if (interface == NULL)
+    interface = default_interface;
+    
+  if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
+    return -1;
+  memset(&ifr, 0, sizeof (ifr));
+  snprintf(ifr.ifr_name, sizeof(ifr.ifr_name), "%s", interface);
+  if (ioctl(s, SIOCGIFMTU, (int8_t *)&ifr) < 0)
+    return -1;
+  
+  close(s);
+  if (debug)
+    printf("DEBUG: MTU %d\n", ifr.ifr_mtu);
+
+  return ifr.ifr_mtu;
+}
+
 unsigned char *thc_get_own_mac(char *interface) {
   int s;
   struct ifreq ifr;
@@ -264,61 +331,80 @@ unsigned char *thc_get_own_ipv6(char *interface, unsigned char *dst, int prefer)
   char *myipv6;
   FILE *f;
   unsigned char ipv6[34] = "", tmpbuf[34], buf[1024], *tmpdst = NULL;
-  int a, b, c, done = 0;
+  int a, b, c, done = 0, picky = 0, orig_prefer = prefer;
   unsigned char tmpd, tmpb;
   char bla[16];  
   
   if (interface == NULL)
     interface = default_interface;
+
+  if (dst != NULL && dst[0] >= 0xf0)
+    dst = NULL;
     
   if (dst != NULL)
     tmpdst = thc_ipv62string(dst);
 
-  if ((f = fopen("/proc/net/if_inet6", "r")) == NULL) {
-    fprintf(stderr, "Error: /proc/net/if_inet6 does not exist, no IPv6 support on your Linux box!\n");
-    exit(-1);
-  }
+  while (done < 2 && picky < 2) {
+    if ((f = fopen("/proc/net/if_inet6", "r")) == NULL) {
+      fprintf(stderr, "Error: /proc/net/if_inet6 does not exist, no IPv6 support on your Linux box!\n");
+      exit(-1);
+    }
 
-  while (done < 2 && fgets(buf, sizeof(buf), f) != NULL) {
-    if (strncmp(interface, &buf[strlen(buf)-strlen(interface)-1], strlen(interface)) == 0) {
-      sscanf(buf, "%s %x %x %x %s", tmpbuf, &a, &b, &c, bla);
-      if ((c != PREFER_HOST || c == prefer) && done == 0) {
-        ipv6[0] = c; // scope type
-        ipv6[1] = b; // netmask
-        memcpy(&ipv6[2], tmpbuf, 32);
-        ipv6[34] = 0;
-        if (dst == NULL)
-          done = 2;
-        else
-          done = 1;
-      }
+    if (picky == 1) {
+      if (prefer == PREFER_GLOBAL)
+        prefer = PREFER_LINK;
+      else
+        prefer = PREFER_GLOBAL;
+    }
+
+    while (done < 2 && fgets(buf, sizeof(buf), f) != NULL) {
+      if (strncmp(interface, &buf[strlen(buf)-strlen(interface)-1], strlen(interface)) == 0) {
+        sscanf(buf, "%s %x %x %x %s", tmpbuf, &a, &b, &c, bla);
+        if (c == prefer && done == 0) {
+          ipv6[0] = c; // scope type
+          ipv6[1] = b; // netmask
+          memcpy(&ipv6[2], tmpbuf, 32);
+          ipv6[34] = 0;
+          if (dst == NULL)
+            done = 2;
+          else
+            done = 1;
+        }
       // if a destination was given, we always prefer the local ip which is in the same subnet of the target
-      if (dst != NULL) {
-        if (strncmp(tmpbuf, tmpdst, b / 8) == 0) {
-          if (b % 8 > 0) {
-            tmpb = tmpbuf[b / 8 + 1] >> (b % 8);
-            tmpd = tmpdst[b / 8 + 1] >> (b % 8);
+        if (dst != NULL) {
+          if (strncmp(tmpbuf, tmpdst, b / 4) == 0 || dst[0] == 0xff) {
+          if (b % 4 > 0) {
+            tmpb = tmpbuf[b / 4 + 1] >> (b % 4);
+            tmpd = tmpdst[b / 4 + 1] >> (b % 4);
             if (tmpb == tmpd) {
               done = 2;
             }
           } else 
-            done = 2;
-          if (done == 2) {
-            if (debug)
-              printf("DEBUG: Found local ipv6 address to destination\n");
-            ipv6[0] = c; // scope type
-            ipv6[1] = b; // netmask
-            memcpy(&ipv6[2], tmpbuf, 32);
-            ipv6[34] = 0;
+              done = 2;
+
+            if (done == 2) {
+              if (debug)
+                printf("DEBUG: Found local ipv6 address to destination\n");
+              ipv6[0] = c; // scope type
+              ipv6[1] = b; // netmask
+              memcpy(&ipv6[2], tmpbuf, 32);
+              ipv6[34] = 0;
+            }
           }
         }
       }
     }
+    fclose(f);
+    picky++;
   }
 
-  fclose(f);
-  if (strlen(&ipv6[2]) == 0)
+  if (strlen(&ipv6[2]) == 0) {
+    fprintf(stderr, "Warning: no IPv6 address on interface defined\n");
     return NULL;
+  }
+
+  if (picky == 2 && orig_prefer != ipv6[0])
+    fprintf(stderr, "Warning: unprefered IPv6 address had to be selected\n");
 
   if (tmpdst != NULL)
     free(tmpdst);
@@ -449,10 +535,10 @@ unsigned char *thc_get_mac(char *interface, unsigned char *src, unsigned char *d
   while (local == 0 && fgets(buf, sizeof(buf), f) != NULL) {
     if (strncmp(interface, &buf[strlen(buf)-strlen(interface)-1], strlen(interface)) == 0) {
       sscanf(buf, "%s %x %x %x %s", tmpbuf, &a, &b, &c, bla);
-      if (strncmp(tmpbuf, tmpdst, b / 8) == 0) {
-        if (b % 8 > 0) {
-          tmpb = tmpbuf[b / 8 + 1] >> (b % 8);
-          tmpd = tmpdst[b / 8 + 1] >> (b % 8);
+      if (strncmp(tmpbuf, tmpdst, b / 4) == 0) {
+        if (b % 4 > 0) {
+          tmpb = tmpbuf[b / 4 + 1] >> (b % 4);
+          tmpd = tmpdst[b / 4 + 1] >> (b % 4);
           if (tmpb == tmpd) {
             local = 1;
           }
@@ -462,6 +548,8 @@ unsigned char *thc_get_mac(char *interface, unsigned char *src, unsigned char *d
     }
   }
   fclose(f);
+  if (debug)
+    printf("DEBUG: is mac local: %d\n", local);
 
   if (! local) {
     if ((f = fopen("/proc/net/ipv6_route", "r")) == NULL) {
@@ -472,10 +560,10 @@ unsigned char *thc_get_mac(char *interface, unsigned char *src, unsigned char *d
       if (strncmp(interface, &buf[strlen(buf)-strlen(interface)-1], strlen(interface)) == 0) {
         sscanf(buf, "%s %x %s %x %s %s", tmpbuf, &b, router1, &a, router2, bla);
         if (b > 0) {
-          if (strncmp(tmpbuf, tmpdst, b / 8) == 0) {
-            if (b % 8 > 0) {
-              tmpb = tmpbuf[b / 8 + 1] >> (b % 8);
-              tmpd = tmpdst[b / 8 + 1] >> (b % 8);
+          if (strncmp(tmpbuf, tmpdst, b / 4) == 0) {
+            if (b % 4 > 0) {
+              tmpb = tmpbuf[b / 4 + 1] >> (b % 4);
+              tmpd = tmpdst[b / 4 + 1] >> (b % 4);
               if (tmpb == tmpd)
                 local = 1;
             } else 
@@ -492,7 +580,7 @@ unsigned char *thc_get_mac(char *interface, unsigned char *src, unsigned char *d
     }
     if (local == 0 && strlen(defaultgw) > 0) {
       if (debug)
-        printf("DEBUG: using default router for %s: %s\n", tmpdst, router2);
+        printf("DEBUG: using default router for %s: %s\n", tmpdst, defaultgw);
       strcpy(tmpdst, defaultgw);
       local = 1;
     }
@@ -501,7 +589,8 @@ unsigned char *thc_get_mac(char *interface, unsigned char *src, unsigned char *d
       return NULL;
     }
   }
-  if ((ret = thc_look_neighborcache(dst)) != NULL)
+
+  if ((ret = thc_look_neighborcache(thc_string2ipv6(tmpdst))) != NULL)
     return ret;
 
   return thc_lookup_ipv6_mac(interface, tmpdst);
@@ -589,8 +678,10 @@ int thc_send_as_fragment6(char *interface, unsigned char *src, unsigned char *ds
   unsigned char buf[frag_len];
   int count, id = time(NULL) % 2000000000, dptr = 0, last_size, run = 0;
 
-  if (frag_len % 8 > 0)
+  if (frag_len % 8 > 0) {
+    fprintf(stderr, "Error: frag_len for thc_send_as_fragment6 must be a multiple of 8, not sending!\n");
     return -1;
+  }
 
   count = data_len / frag_len;
   if (data_len % frag_len > 0) {
@@ -1062,12 +1153,51 @@ int thc_add_hdr_route(unsigned char *pkt, int *pkt_len, unsigned char **routers,
   hdr->length += nehdr->data_len + 2;
   *pkt_len += nehdr->data_len + 2;
 
-  if (i > 0)
+  if (i > 0 && routerptr > 0)
     hdr->final_dst = nehdr->data + 6 + (i - 1)*16;
 
   return 0;
 }
 
+int thc_add_hdr_mobileroute(unsigned char *pkt, int *pkt_len, unsigned char *dst) {
+  thc_ipv6_hdr *hdr = (thc_ipv6_hdr *) pkt;
+  thc_ipv6_ext_hdr *ehdr = (thc_ipv6_ext_hdr *) hdr->final, *nehdr = malloc(sizeof(thc_ipv6_ext_hdr));
+  unsigned char *buf;
+
+  if (nehdr == NULL || hdr == NULL)
+    return -1;
+
+  if (ehdr == NULL) {
+    hdr->next = NXT_ROUTE;
+    hdr->next_segment = (char *) nehdr;
+  } else {
+    ehdr->next = NXT_ROUTE;
+    ehdr->next_segment = (char *) nehdr;
+  }
+  hdr->final = (char *) nehdr;
+  hdr->final_type = NXT_ROUTE;
+
+  if ((buf = malloc(16 + 2 + 4)) == NULL)
+    return -1;
+  memset(buf, 0, 16 + 2 + 4);
+  // byte 0 = type; 1 = routers to do; byte 2 reserved; bytes 3-5: loose source routing
+  buf[0] = 2;
+  buf[1] = 1;
+  memcpy(buf+6, dst, 16);
+
+  nehdr->next_segment = NULL;
+  nehdr->next = NXT_NONXT;
+  nehdr->data = buf;
+  nehdr->data_len = 16 + 2 + 4;
+  nehdr->length = 2;
+  hdr->length += nehdr->data_len + 2;
+  *pkt_len += nehdr->data_len + 2;
+
+  hdr->final_dst = nehdr->data + 6;
+
+  return 0;
+}
+
 int thc_add_hdr_fragment(unsigned char *pkt, int *pkt_len, int offset, char more_frags, unsigned int id) {
   thc_ipv6_hdr *hdr = (thc_ipv6_hdr *) pkt;
   thc_ipv6_ext_hdr *ehdr = (thc_ipv6_ext_hdr *) hdr->final, *nehdr = malloc(sizeof(thc_ipv6_ext_hdr));
@@ -1184,6 +1314,70 @@ int thc_add_icmp6(unsigned char *pkt, int *pkt_len, int type, int code, unsigned
   return 0;
 }
 
+int thc_add_tcp(unsigned char *pkt, int *pkt_len, unsigned short int sport, unsigned short int dport, unsigned int sequence, unsigned int ack, unsigned char flags, unsigned short int window, unsigned short int urgent, char *option, int option_len, char *data, int data_len) {
+  thc_ipv6_hdr *hdr = (thc_ipv6_hdr *) pkt;
+  thc_tcp_hdr *ihdr = malloc(sizeof(thc_tcp_hdr));
+  thc_ipv6_ext_hdr *ehdr;
+  int i = option_len;
+
+  if (ihdr == NULL)
+    return -1;
+  memset(ihdr, 0, sizeof(thc_tcp_hdr));
+
+  if (hdr->final != NULL) {
+    ehdr = (thc_ipv6_ext_hdr *) hdr->final;
+    ehdr->next_segment = (char *) ihdr;
+    ehdr->next=NXT_TCP;
+  } else {
+    hdr->next_segment = (char *) ihdr;
+    hdr->next=NXT_TCP;
+  }
+  hdr->final = (char *) ihdr;
+  hdr->final_type=NXT_TCP;
+  
+  ihdr->sport = sport;
+  ihdr->dport = dport;
+  ihdr->sequence = sequence;
+  ihdr->ack = ack;
+  ihdr->flags = flags;
+  ihdr->window = window;
+  ihdr->urgent = urgent;
+
+//  if (checksum == 0) {
+    ihdr->checksum = DO_CHECKSUM;
+//  } else
+//    ihdr->checksum = checksum;
+
+  if (data_len > 0 && data != NULL) {
+    ihdr->data = malloc(data_len);
+    ihdr->data_len = data_len;
+    memcpy(ihdr->data, data, data_len);
+  } else {
+    ihdr->data = NULL;
+    ihdr->data_len = 0;
+  }
+
+  if (option_len > 0 && option != NULL) {
+    if ((i = option_len) % 4 > 0)
+      option_len = (((option_len / 4) + 1) * 4);
+    ihdr->option = malloc(option_len);
+    ihdr->option_len = option_len;
+    memcpy(ihdr->option, option, i);
+  } else {
+    ihdr->option = NULL;
+    ihdr->option_len = 0;
+  }
+
+  i = (20 + option_len) / 4;
+  ihdr->length = ((i % 16) * 16) + (i / 16);
+
+  hdr->length += data_len + 20 + option_len;
+  *pkt_len += data_len + 20 + option_len;
+
+  return 0;
+}
+
+
 int thc_add_data6(unsigned char *pkt, int *pkt_len, unsigned char type, unsigned char *data, int data_len) {
   thc_ipv6_hdr *hdr = (thc_ipv6_hdr *) pkt;
   thc_ipv6_ext_hdr *ehdr = (thc_ipv6_ext_hdr *) hdr->final, *nehdr = malloc(sizeof(thc_ipv6_ext_hdr));
@@ -1223,7 +1417,8 @@ int thc_generate_pkt(char *interface, unsigned char *srcmac, unsigned char *dstm
   thc_ipv6_hdr *hdr = (thc_ipv6_hdr*) pkt;
   thc_ipv6_ext_hdr *ehdr;
   thc_icmp6_hdr *ihdr;
-  char *next, *mysrcmac, *mydstmac, *last_type;
+  thc_tcp_hdr *thdr;
+  char *next, *mysrcmac, *mydstmac, *last_type, *checksum_src;
   int type, bufptr, do_checksum = 0, offset = 0;
 
   if (pkt == NULL || hdr->pkt != NULL || (hdr->pkt = malloc(*pkt_len + 14)) == NULL)
@@ -1270,22 +1465,27 @@ int thc_generate_pkt(char *interface, unsigned char *srcmac, unsigned char *dstm
   hdr->pkt[5+offset] = hdr->length % 256;
   hdr->pkt[6+offset] = hdr->next;
   last_type = &hdr->pkt[7+offset];
-  hdr->pkt[21] = hdr->ttl;
+  hdr->pkt[7+offset] = hdr->ttl;
   memcpy(&hdr->pkt[8+offset], hdr->src, 16);
   memcpy(&hdr->pkt[24+offset], hdr->dst, 16);
   
   next = hdr->next_segment;
   type = hdr->next;
   bufptr = 40+offset;
+  checksum_src = hdr->original_src;
 
   while (type == NXT_HDR || type == NXT_ROUTE || type == NXT_FRAG
-         || type == NXT_OPTS || type == NXT_INVALID) {
+         || type == NXT_OPTS || type == NXT_INVALID || type == NXT_IGNORE) {
     ehdr = (thc_ipv6_ext_hdr *) next;
     hdr->pkt[bufptr] = ehdr->next;
     hdr->pkt[bufptr+1] = ehdr->length;
     last_type = &hdr->pkt[bufptr];
-    if (ehdr->data != NULL && ehdr->data_len > 0)
+    if (ehdr->data != NULL && ehdr->data_len > 0) {
       memcpy(&hdr->pkt[bufptr+2], ehdr->data, ehdr->data_len);
+      if (type == NXT_OPTS && hdr->pkt[bufptr+2] == 0xc9) { // mobile home address option
+        checksum_src = &hdr->pkt[bufptr+4];
+      }
+    }
     bufptr += 2 + ehdr->data_len;
     next = ehdr->next_segment;
     type = ehdr->next;
@@ -1311,18 +1511,74 @@ int thc_generate_pkt(char *interface, unsigned char *srcmac, unsigned char *dstm
         if (ihdr->data != NULL && ihdr->data_len > 0)
           memcpy(&hdr->pkt[bufptr+8], ihdr->data, ihdr->data_len);
         if (do_checksum) {
-          ihdr->checksum = checksum_pseudo_header(hdr->original_src, hdr->final_dst, NXT_ICMP6, &hdr->pkt[bufptr], 8 + ihdr->data_len);
+//memcpy( hdr->final_dst, hdr->pkt + 38, 16);
+          ihdr->checksum = checksum_pseudo_header(checksum_src, hdr->final_dst, NXT_ICMP6, &hdr->pkt[bufptr], 8 + ihdr->data_len);
+/*
+printf("\n");
+thc_dump_data((unsigned char *)hdr->pkt + 22, 16,"packet     source");
+thc_dump_data((unsigned char *)checksum_src, 16, "original   source");
+thc_dump_data((unsigned char *)hdr->final_dst, 16,    "final destination");
+thc_dump_data((unsigned char *)hdr->pkt + 38, 16,    "pkt   destination");
+printf("\n");
+*/
           hdr->pkt[bufptr+2] = ihdr->checksum / 256;
           hdr->pkt[bufptr+3] = ihdr->checksum % 256;
           do_checksum = 0;
         }
         bufptr += 8 + ihdr->data_len;
       break;
+    case NXT_TCP:
+        thdr = (thc_tcp_hdr *) next;
+        if (thdr->checksum == DO_CHECKSUM) {
+          thdr->checksum = 0;
+          do_checksum = 1;
+        }
+        hdr->pkt[bufptr] = thdr->sport / 256;
+        hdr->pkt[bufptr + 1] = thdr->sport % 256;
+        hdr->pkt[bufptr + 2] = thdr->dport / 256;
+        hdr->pkt[bufptr + 3] = thdr->dport % 256;
+        hdr->pkt[bufptr + 4] = thdr->sequence / 16777216;
+        hdr->pkt[bufptr + 5] = (thdr->sequence % 16777216) / 65536;
+        hdr->pkt[bufptr + 6] = (thdr->sequence % 65536) / 256;
+        hdr->pkt[bufptr + 7] = thdr->sequence % 256;
+        hdr->pkt[bufptr + 8] = thdr->ack / 16777216;
+        hdr->pkt[bufptr + 9] = (thdr->ack % 16777216) / 65536;
+        hdr->pkt[bufptr + 10] = (thdr->ack % 65536) / 256;
+        hdr->pkt[bufptr + 11] = thdr->ack % 256;
+        hdr->pkt[bufptr + 12] = thdr->length;
+        hdr->pkt[bufptr + 13] = thdr->flags;
+        hdr->pkt[bufptr + 14] = thdr->window % 256;
+        hdr->pkt[bufptr + 15] = thdr->window / 256;
+        hdr->pkt[bufptr + 18] = thdr->urgent % 256;
+        hdr->pkt[bufptr + 19] = thdr->urgent / 256;
+        
+        if (thdr->option != NULL && thdr->option_len > 0)
+          memcpy(&hdr->pkt[bufptr+20], thdr->option, thdr->option_len);
+        if (thdr->data != NULL && thdr->data_len > 0)
+          memcpy(&hdr->pkt[bufptr+20+thdr->option_len], thdr->data, thdr->data_len);
+        if (do_checksum) {
+//memcpy( hdr->final_dst, hdr->pkt + 38, 16);
+          thdr->checksum = checksum_pseudo_header(checksum_src, hdr->final_dst, NXT_TCP, &hdr->pkt[bufptr], 20 + thdr->option_len + thdr->data_len);
+/*
+printf("\n");
+thc_dump_data((unsigned char *)hdr->pkt + 22, 16,"packet     source");
+thc_dump_data((unsigned char *)checksum_src, 16, "original   source");
+thc_dump_data((unsigned char *)hdr->final_dst, 16,    "final destination");
+thc_dump_data((unsigned char *)hdr->pkt + 38, 16,    "pkt   destination");
+printf("\n");
+*/
+          hdr->pkt[bufptr+16] = thdr->checksum / 256;
+          hdr->pkt[bufptr+17] = thdr->checksum % 256;
+          do_checksum = 0;
+        }
+        bufptr += 20 + thdr->option_len + thdr->data_len;
+      
+      break;
     case NXT_DATA:
         ehdr = (thc_ipv6_ext_hdr *) next;
         memcpy(&hdr->pkt[bufptr], ehdr->data, ehdr->data_len);
         if (ehdr->next == NXT_MIPV6) {
-          do_checksum = checksum_pseudo_header(hdr->original_src, hdr->final_dst, NXT_MIPV6, &hdr->pkt[bufptr], ehdr->data_len);
+          do_checksum = checksum_pseudo_header(checksum_src, hdr->final_dst, NXT_MIPV6, &hdr->pkt[bufptr], ehdr->data_len);
           hdr->pkt[bufptr+4] = do_checksum / 256;
           hdr->pkt[bufptr+5] = do_checksum % 256;
         }
@@ -1350,7 +1606,7 @@ int thc_send_pkt(char *interface, unsigned char *pkt, int *pkt_len) {
   struct sockaddr sa;
   thc_ipv6_hdr *hdr = (thc_ipv6_hdr*) pkt;
   
-  if (pkt == NULL || hdr->pkt == NULL || hdr->pkt_len < 1)
+  if (pkt == NULL || hdr->pkt == NULL || hdr->pkt_len < 1 || hdr->pkt_len > 65535)
     return -1;
 
   if (interface == NULL)
@@ -1361,6 +1617,9 @@ int thc_send_pkt(char *interface, unsigned char *pkt, int *pkt_len) {
 
   if (debug)
     thc_dump_data(hdr->pkt, hdr->pkt_len, "Sent Packet");
+
+  if (hdr->pkt_len > thc_get_mtu(interface))
+    fprintf(stderr, "Warning: packet size is larger than MTU of interface (%d > %d)!\n", hdr->pkt_len, thc_get_mtu(interface));
   
   return sendto(thc_socket, hdr->pkt, hdr->pkt_len, 0, &sa, sizeof(sa));
 }
@@ -1379,6 +1638,7 @@ unsigned char *thc_destroy_packet(unsigned char *pkt) {
   thc_ipv6_hdr *hdr = (thc_ipv6_hdr*) pkt;
   thc_ipv6_ext_hdr *ehdr;
   thc_icmp6_hdr *ihdr;
+  thc_tcp_hdr *thdr;
   
   ptrs[iptr] = pkt;
   iptr++;
@@ -1389,7 +1649,7 @@ unsigned char *thc_destroy_packet(unsigned char *pkt) {
     free(hdr->pkt);
 
   while (type == NXT_HDR || type == NXT_ROUTE || type == NXT_FRAG
-         || type == NXT_OPTS || type == NXT_INVALID) {
+         || type == NXT_OPTS || type == NXT_INVALID || type == NXT_IGNORE) {
     ehdr = (thc_ipv6_ext_hdr *) next;
     ptrs[iptr] = ehdr->data;
     iptr++;
@@ -1405,6 +1665,13 @@ unsigned char *thc_destroy_packet(unsigned char *pkt) {
         ptrs[iptr] = ihdr->data;
         iptr++;
       break;
+    case NXT_TCP:
+        thdr = (thc_tcp_hdr *) next;
+        ptrs[iptr] = thdr->option;
+        iptr++;
+        ptrs[iptr] = thdr->data;
+        iptr++;
+      break;
     case NXT_DATA:
         ehdr = (thc_ipv6_ext_hdr *) next;
         ptrs[iptr] = ehdr->data;
@@ -1528,7 +1795,7 @@ thc_cga_hdr *thc_generate_cga(unsigned char *prefix, thc_key_t *key, unsigned ch
    thc_cga_hdr *cga_hdr;
    unsigned char md_value[EVP_MAX_MD_SIZE];
    unsigned char *p, *tmp;
-   int klen, rand_fd, cgasize;
+   int klen, rand_fd, cgasize, ignore;
 
 
     if((cga_hdr = (thc_cga_hdr *)malloc(sizeof(thc_cga_hdr))) == NULL)
@@ -1545,7 +1812,7 @@ thc_cga_hdr *thc_generate_cga(unsigned char *prefix, thc_key_t *key, unsigned ch
     
     if((rand_fd = open("/dev/urandom", O_RDONLY)) < 0 )
         { printf("Cannot open source of randomness!\n"); return NULL; }
-    read(rand_fd, cga_hdr->modifier, 16);
+    ignore = read(rand_fd, cga_hdr->modifier, 16);
     close(rand_fd);
 
     /* DER-encode public key */
@@ -1636,7 +1903,7 @@ thc_rsa_hdr *thc_generate_rsa(char *data2sign, int data2sign_len, thc_cga_hdr *c
 {
    thc_rsa_hdr *rsa_hdr;
    unsigned char md_value[EVP_MAX_MD_SIZE], hash[20];
-   int rsa_hdr_len, sign_len, fd;
+   int rsa_hdr_len, sign_len, fd, ignore;
          
    if((rsa_hdr = (thc_rsa_hdr *)malloc(sizeof(thc_rsa_hdr))) == NULL)
      return NULL;
@@ -1654,7 +1921,7 @@ thc_rsa_hdr *thc_generate_rsa(char *data2sign, int data2sign_len, thc_cga_hdr *c
    else
      {
        fd = open("/dev/urandom", O_RDONLY);
-       read(fd, hash, 20);
+       ignore = read(fd, hash, 20);
        close(fd);
      }
 
