@@ -1,5 +1,5 @@
 /*
- * (c) 2006 by van Hauser / THC
+ * (c) 2010 by van Hauser / THC
  *
  * THC IPv6 Attack Library Header Files
  *
@@ -12,26 +12,40 @@
 #include <pcap.h>
 #include <openssl/rsa.h>
 
-#define VERSION 	"v0.7"
+#define VERSION 	"v1.1"
 #define AUTHOR 		"van Hauser / THC <vh@thc.org>"
 #define RESOURCE	"www.thc.org"
 
 //#define MULTICAST_ALL_NODES
 
-#define ICMP6_PINGREQUEST 128
-#define ICMP6_PINGREPLY   129
-#define ICMP6_NEIGHBORADV 136
-#define ICMP6_NEIGHBORSOL 135
-#define ICMP6_ROUTERADV   134
-#define ICMP6_ROUTERSOL   133
 #define ICMP6_UNREACH     1
 #define ICMP6_TOOBIG      2
 #define ICMP6_TTLEXEED    3
 #define ICMP6_PARAMPROB   4
-#define ICMP6_REDIR       137    
+#define ICMP6_PINGREQUEST 128
+#define ICMP6_PINGREPLY   129
 #define ICMP6_MLD_QUERY   130
 #define ICMP6_MLD_REPORT  131
 #define ICMP6_MLD_DONE    132
+#define ICMP6_ROUTERSOL   133
+#define ICMP6_ROUTERADV   134
+#define ICMP6_NEIGHBORSOL 135
+#define ICMP6_NEIGHBORADV 136
+#define ICMP6_REDIR       137    
+#define ICMP6_INFOREQUEST 139
+#define ICMP6_INFOREPLY   140
+#define ICMP6_INVNEIGHBORSOL 141
+#define ICMP6_INVNEIGHBORADV 142
+#define ICMP6_MOBILE_PREFIXSOL 146
+#define ICMP6_MOBILE_PREFIXADV 147
+#define ICMP6_CERTPATHSOL 148
+#define ICMP6_CERTPATHADV 149
+#define ICMP6_MLD_ROUTERADV 151
+#define ICMP6_MLD_ROUTERSOL 152
+#define ICMP6_MLD_ROUTERTERMINATION 153
+#define ICMP6_ROUTERPROXYSOL 154
+#define ICMP6_ROUTERPROXYADV 155
+
 
 #define ICMP6_NEIGHBORADV_ROUTER   0x80000000
 #define ICMP6_NEIGHBORADV_SOLICIT  0x40000000
@@ -49,6 +63,7 @@ extern void thc_pcap_close(pcap_t *p);
 extern char *thc_resolve6(unsigned char *target);
 extern unsigned char *thc_lookup_ipv6_mac(char *interface, unsigned char *dst);
 extern unsigned char *thc_get_own_mac(char *interface);
+extern int thc_get_mtu(char *interface);
 extern unsigned char *thc_get_own_ipv6(char *interface, unsigned char *dst, int prefer);
 extern unsigned char *thc_get_multicast_mac(unsigned char *dst);
 extern unsigned char *thc_get_mac(char *interface, unsigned char *src, unsigned char *dst);
@@ -70,11 +85,13 @@ extern char *thc_create_ipv6(char *interface, int prefer, int *pkt_len, unsigned
 
 extern int thc_add_hdr_misc(unsigned char *pkt, int *pkt_len, unsigned char type, int len, unsigned char *buf, int buflen);
 extern int thc_add_hdr_route(unsigned char *pkt, int *pkt_len, unsigned char **routers, unsigned char routerptr);
+extern int thc_add_hdr_mobileroute(unsigned char *pkt, int *pkt_len, unsigned char *dst);
 extern int thc_add_hdr_fragment(unsigned char *pkt, int *pkt_len, int offset, char more_frags, unsigned int id);
 extern int thc_add_hdr_dst(unsigned char *pkt, int *pkt_len, unsigned char *buf, int buflen);
 extern int thc_add_hdr_hopbyhop(unsigned char *pkt, int *pkt_len, unsigned char *buf, int buflen);
 extern int thc_add_hdr_nonxt(unsigned char *pkt, int *pkt_len, int hdropt);
 extern int thc_add_icmp6(unsigned char *pkt, int *pkt_len, int type, int code, unsigned int flags, unsigned char *data, int data_len, int checksum);
+extern int thc_add_tcp(unsigned char *pkt, int *pkt_len, unsigned short int sport, unsigned short int dport, unsigned int sequence, unsigned int ack, unsigned char flags, unsigned short int window, unsigned short int urgent, char *option, int option_len, char *data, int data_len);
 extern int thc_add_data6(unsigned char *pkt, int *pkt_len, unsigned char type, unsigned char *data, int data_len);
 
 extern int thc_generate_and_send_pkt(char *interface, unsigned char *srcmac, unsigned char *dstmac, unsigned char *pkt, int *pkt_len);
@@ -84,12 +101,13 @@ unsigned char *thc_destroy_packet(unsigned char *pkt);
 
 extern int thc_open_ipv6();
 extern int checksum_pseudo_header(unsigned char *src, unsigned char *dst, unsigned char type, unsigned char *data, int length);
+extern int calculate_checksum(unsigned char *data, int data_len);
 void thc_dump_data(unsigned char *buf, int len, char *text);
 unsigned char *thc_ipv62string(unsigned char *ipv6);
 unsigned char *thc_string2ipv6(unsigned char *string);
 unsigned char *thc_string2notation(unsigned char *string);
 unsigned char *thc_memstr(char *haystack, char *needle, int haystack_length, int needle_length);
-
+void thc_notation2beauty(unsigned char *ipv6);
 
 #define DO_CHECKSUM 0xffff
 
@@ -116,6 +134,15 @@ unsigned char *thc_memstr(char *haystack, char *needle, int haystack_length, int
 
 #define IPV6_FRAME_TYPE 0x86dd
 
+#define TCP_CWR 128
+#define TCP_ECN 64
+#define TCP_URG 32
+#define TCP_ACK 16
+#define TCP_PSH 8
+#define TCP_RST 4
+#define TCP_SYN 2
+#define TCP_FIN 1
+
 typedef struct {
   unsigned char dst[6];
   unsigned char src[6];
@@ -158,6 +185,22 @@ typedef struct {
   int           data_len;
 } thc_icmp6_hdr;
 
+typedef struct {
+  unsigned short int sport;
+  unsigned short int dport;
+  unsigned int  sequence;
+  unsigned int  ack;
+  unsigned char length;
+  unsigned char flags;
+  unsigned short int window;
+  unsigned short int checksum;
+  unsigned short int urgent;
+  unsigned char *option;
+  int           option_len;
+  unsigned char *data;
+  int           data_len;
+} thc_tcp_hdr;
+
 typedef struct {
   char *next_segment;
   char dummy[8];
